#include <map>
#include <list>
#include <iostream>

struct IteradorConteo {
    IteradorConteo(int conteo,
                    std::list<std::string>::iterator iterador) :
        conteo(conteo),
        iterador(iterador)
    {}
    int conteo;
    std::list<std::string>::iterator iterador;
};
typedef std::map<std::string, IteradorConteo> ConteoPorPalabra;
typedef std::map<int, std::list<std::string> > PalabrasPorConteo;

// Función auxiliar para imprimir las palabras según se número de repeticiones
void imprimir(std::map<int, std::list<std::string> > &tabla)
{
    for (std::map<int, std::list<std::string> >::const_iterator i = 
             tabla.begin();
         i != tabla.end(); ++i) {
        std::cout << "Palabras con " << i->first 
                  << " repeticiones:" << std::endl;
        for (std::list<std::string>::const_iterator j = i->second.begin();
             j != i->second.end(); ++j) {
            std::cout << *j << ' ';
        }
        std::cout << std::endl;
    }
}

int main(int argc, char *argv[])
{
    ConteoPorPalabra conteoPorPalabra;
    PalabrasPorConteo palabrasPorConteo;

    // Bucle de lectura
    while (!std::cin.eof()) {
        std::string palabra;
        std::cin >> palabra;
        if (!std::cin.fail()) {
            ConteoPorPalabra::iterator conteo = conteoPorPalabra.find(palabra);
            if (conteo == conteoPorPalabra.end()) {
                // Primera aparición de esta palabra.
                std::list<std::string> &lista = palabrasPorConteo[1];
                lista.push_back(palabra);
                conteoPorPalabra.insert
                    (std::make_pair(palabra, 
                                    IteradorConteo(1, --lista.end())));
            } else {
                // Incrementamos la cuenta actual
                int &cuenta = ++conteo->second.conteo;
                // Ahora sacamos la palabra de su lista actual y la llevamos
                // a la lista que le corresponde.
                std::list<std::string> &nuevaLista = palabrasPorConteo[cuenta];
                // Buscamos la lista anterior con find porque queremos un 
                // iterador a la misma.
                PalabrasPorConteo::iterator listaAnterior = 
                    palabrasPorConteo.find(cuenta - 1);
                nuevaLista.splice(nuevaLista.end(), listaAnterior->second,
                                  conteo->second.iterador);
                // Eliminamos la lista anterior del map palabrasPorConteo 
                // si esta se ha quedado vacía. Para eso queríamos un iterador
                // a la lista.
                if (listaAnterior->second.empty())
                    palabrasPorConteo.erase(listaAnterior);
                
                // Por último actualizamos el iterador que apunta a la cadena
                // en la lista que le corresponde.
                conteo->second.iterador = --nuevaLista.end();
            }
        }

        // Para ver al evolución del resultado vamos a imprimir las listas
        // de palabras con cada número de repeticiones en cada iteración.
        std::cout << "Leido: " << palabra << std::endl;
        imprimir(palabrasPorConteo);
    }
}
